/////////////////////////////////////////////////////////////////////////
// $Id: cdrom.cc,v 1.25.2.1 2001/12/10 18:38:57 bdenney Exp $
/////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2001  MandrakeSoft S.A.
//
//    MandrakeSoft S.A.
//    43, rue d'Aboukir
//    75002 Paris - France
//    http://www.linux-mandrake.com/
//    http://www.mandrakesoft.com/
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA


// These are the low-level CDROM functions which are called
// from 'harddrv.cc'.  They effect the OS specific functionality
// needed by the CDROM emulation in 'harddrv.cc'.  Mostly, just
// ioctl() calls and such.  Should be fairly easy to add support
// for your OS if it is not supported yet.


#include "bochs.h"

#define LOG_THIS /* no SMF tricks here, not needed */

#include <descrip.h>
#include <devdef.h>
#include <dvidef.h>
#include <iodef.h>
#include <mntdef.h>
#include <ssdef.h>
#include <starlet.h>

struct s_iosb {
	unsigned short status;
	unsigned long count;
	unsigned short other;
};
struct s_dvi_list {
	unsigned short bufflen;
	unsigned short code;
	unsigned long buffaddr;
	unsigned long retlen;
	unsigned long term;
};
struct s_mnt_list {
	unsigned short opt_bufflen;
	unsigned short opt_code;
	unsigned long opt_buffaddr;
	unsigned long opt_retlen;
	unsigned short nam_bufflen;
	unsigned short nam_code;
	unsigned long nam_buffaddr;
	unsigned long nam_retlen;
	unsigned long term;
};

#define BX_CD_FRAMESIZE 2048
#define VMS_BLOCK_SIZE 512

cdrom_interface::cdrom_interface(char *dev)
{
  put("CD");
  settype(CDLOG);

  if ( dev == NULL )
    path = NULL;
  else {
    path = strdup(dev);
    mounted = false;
    assigned = false;
    maxblock = 0;
    BX_INFO(("Init, file = '%s'\n",dev));
  }
}

void
cdrom_interface::init(void) {
  BX_DEBUG(("Init $Id: cdrom.cc,v 1.25.2.1 2001/12/10 18:38:57 bdenney Exp $"));
  BX_INFO(("file = '%s'",path));
}

cdrom_interface::~cdrom_interface(void)
{
  if (mounted || assigned)
    eject_cdrom();
  if (path)
    free(path);
	BX_DEBUG(("Exit"));
}

  bool
cdrom_interface::insert_cdrom()
{
	unsigned long status;
	struct dsc$descriptor devnam;
	struct s_mnt_list mnt_list;
	unsigned long mnt_flags[2];
	unsigned long mnt_retlen[2];
	struct s_dvi_list dvi_list;
	unsigned long dvi_retlen;
	struct s_iosb iosb;

// check mounted/assigned

	if (mounted || assigned) {
		BX_PANIC(("insert_cdrom: already assigned or mounted.\n"));
	}

	devnam.dsc$b_class = DSC$K_CLASS_S;
	devnam.dsc$w_length = strlen(path);
	devnam.dsc$a_pointer = path;

// mount it foreign

	mnt_flags[0] = MNT$M_FOREIGN | MNT$M_NOASSIST;
	mnt_flags[1] = 0;

	mnt_list.opt_bufflen = 8;
	mnt_list.opt_code = MNT$_FLAGS;
	mnt_list.opt_buffaddr = (long)&mnt_flags[0];
	mnt_list.opt_retlen = (long)&mnt_retlen[0];
	mnt_list.nam_bufflen = devnam.dsc$w_length;
	mnt_list.nam_code = MNT$_DEVNAM;
	mnt_list.nam_buffaddr = (long)devnam.dsc$a_pointer;
	mnt_list.nam_retlen = 0;
	mnt_list.term = 0;

	status = sys$mount(&mnt_list);
	if (status != SS$_NORMAL) {
		BX_INFO(("sys$mount failed status %08x\n", status));
		return(false);
	}
	mounted = true;

// assign channel

	status = sys$assign( &devnam, &chan, 0, 0, 0);
	if (status != SS$_NORMAL) {
		BX_INFO(("sys$assgn failed status %08x\n", status));
		return(false);
	}
	assigned = true;

// save maxblock

	dvi_list.bufflen = 4;
	dvi_list.code = DVI$_MAXBLOCK;
	dvi_list.buffaddr = (long)&maxblock;
	dvi_list.retlen = (long)&dvi_retlen;
	dvi_list.term = 0;
	status = sys$getdviw( 0, chan, 0, &dvi_list, &iosb, 0, 0, 0);
	if ((status != SS$_NORMAL) || (iosb.status != SS$_NORMAL)) {
		BX_INFO(("sys$getdviw failed status %08x iosb %04x\n"
			, status, iosb.status));
		return(false);
	}

	BX_DEBUG(("maxblock = %u\n", maxblock));

	return(true);
}


  void
cdrom_interface::eject_cdrom()
{
	unsigned long status;
	struct dsc$descriptor devnam;

	devnam.dsc$b_class = DSC$K_CLASS_S;
	devnam.dsc$w_length = strlen(path);
	devnam.dsc$a_pointer = path;

	if (assigned) {
		status = sys$dassgn(chan);
		if (status != SS$_NORMAL) {
			BX_INFO(("sys$dassgn failed status %08x\n", status));
		} else {
			assigned = false;
			chan = 0;
			maxblock = 0;
		}
	}

	if (mounted) {
		status = sys$dismou(&devnam, 0);
		if (status != SS$_NORMAL) {
			BX_INFO(("sys$dismou failed status %08x\n", status));
		} else {
			mounted = false;
		}
	}

}


  bool
cdrom_interface::read_toc(uint8* buf, int* length, bool msf, int start_track)
{
  BX_INFO(("read_toc: not supported (yet) in VMS port.\n"));
  return(false); // OS not supported yet, return false always.
}


  uint32
cdrom_interface::capacity()
{
	if (!mounted || !assigned) {
		BX_PANIC(("cdrom: capacity: not mounted or not assigned.\n"));
	}

	BX_DEBUG(("sectors = %u\n", maxblock/(BX_CD_FRAMESIZE/VMS_BLOCK_SIZE)));

	return( maxblock / ( BX_CD_FRAMESIZE / VMS_BLOCK_SIZE ) );
}

  void
cdrom_interface::read_block(uint8* buf, int lba)
{
	unsigned long status;
	struct s_iosb iosb;

	status = sys$qiow( 0, chan, IO$_READVBLK, &iosb, 0, 0
			, buf, BX_CD_FRAMESIZE
			, lba * ( BX_CD_FRAMESIZE / VMS_BLOCK_SIZE )
			, 0, 0, 0);

	if ((status != SS$_NORMAL) || (iosb.status != SS$_NORMAL)) {
		BX_PANIC(("sys$qiow failed with status %08x iosb status %04x\n"
			, status, iosb.status));
	}
	BX_DEBUG(("read %u %u\n", lba, status));
}
